@@ -19,27 +19,58 @@
  *	Boston, MA 02110-1301, USA
  */
 
-#include "flegita-option-widgets.h"
+#include <glib.h>
+#include <glib-object.h>
+#include <gnome-scan.h>
 #include <gtk/gtk.h>
 #include <stdlib.h>
 #include <string.h>
 #include <gdk/gdk.h>
 #include <glib/gi18n-lib.h>
-#include <gnome-scan-common.h>
-#include "flegita-options.h"
 
 
 #define FLEGITA_TYPE_FORMAT_COLUMNS (flegita_format_columns_get_type ())
 
+#define FLEGITA_TYPE_OUTPUT_FILENAME_WIDGET (flegita_output_filename_widget_get_type ())
+#define FLEGITA_OUTPUT_FILENAME_WIDGET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FLEGITA_TYPE_OUTPUT_FILENAME_WIDGET, FlegitaOutputFilenameWidget))
+#define FLEGITA_OUTPUT_FILENAME_WIDGET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FLEGITA_TYPE_OUTPUT_FILENAME_WIDGET, FlegitaOutputFilenameWidgetClass))
+#define FLEGITA_IS_OUTPUT_FILENAME_WIDGET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FLEGITA_TYPE_OUTPUT_FILENAME_WIDGET))
+#define FLEGITA_IS_OUTPUT_FILENAME_WIDGET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FLEGITA_TYPE_OUTPUT_FILENAME_WIDGET))
+#define FLEGITA_OUTPUT_FILENAME_WIDGET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FLEGITA_TYPE_OUTPUT_FILENAME_WIDGET, FlegitaOutputFilenameWidgetClass))
+
+typedef struct _FlegitaOutputFilenameWidget FlegitaOutputFilenameWidget;
+typedef struct _FlegitaOutputFilenameWidgetClass FlegitaOutputFilenameWidgetClass;
+typedef struct _FlegitaOutputFilenameWidgetPrivate FlegitaOutputFilenameWidgetPrivate;
+#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
+#define _g_hash_table_unref0(var) ((var == NULL) ? NULL : (var = (g_hash_table_unref (var), NULL)))
+
+#define FLEGITA_TYPE_OPTION_OUTPUT_FILENAME (flegita_option_output_filename_get_type ())
+#define FLEGITA_OPTION_OUTPUT_FILENAME(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), FLEGITA_TYPE_OPTION_OUTPUT_FILENAME, FlegitaOptionOutputFilename))
+#define FLEGITA_OPTION_OUTPUT_FILENAME_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), FLEGITA_TYPE_OPTION_OUTPUT_FILENAME, FlegitaOptionOutputFilenameClass))
+#define FLEGITA_IS_OPTION_OUTPUT_FILENAME(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FLEGITA_TYPE_OPTION_OUTPUT_FILENAME))
+#define FLEGITA_IS_OPTION_OUTPUT_FILENAME_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FLEGITA_TYPE_OPTION_OUTPUT_FILENAME))
+#define FLEGITA_OPTION_OUTPUT_FILENAME_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), FLEGITA_TYPE_OPTION_OUTPUT_FILENAME, FlegitaOptionOutputFilenameClass))
+
+typedef struct _FlegitaOptionOutputFilename FlegitaOptionOutputFilename;
+typedef struct _FlegitaOptionOutputFilenameClass FlegitaOptionOutputFilenameClass;
+#define _g_free0(var) (var = (g_free (var), NULL))
+#define _gtk_tree_path_free0(var) ((var == NULL) ? NULL : (var = (gtk_tree_path_free (var), NULL)))
+
 typedef enum  {
 	FLEGITA_FORMAT_COLUMNS_ICON,
 	FLEGITA_FORMAT_COLUMNS_SUFFIX,
 	FLEGITA_FORMAT_COLUMNS_N
 } FlegitaFormatColumns;
 
+struct _FlegitaOutputFilenameWidget {
+	GnomeScanOptionWidget parent_instance;
+	FlegitaOutputFilenameWidgetPrivate * priv;
+};
 
+struct _FlegitaOutputFilenameWidgetClass {
+	GnomeScanOptionWidgetClass parent_class;
+};
 
-GType flegita_format_columns_get_type (void);
 struct _FlegitaOutputFilenameWidgetPrivate {
 	GtkEntry* entry;
 	GtkFileChooser* filechooser;
@@ -49,20 +80,32 @@ struct _FlegitaOutputFilenameWidgetPrivate {
 	gboolean inhibit;
 };
 
+
+static gpointer flegita_output_filename_widget_parent_class = NULL;
+
+GType flegita_format_columns_get_type (void);
+GType flegita_output_filename_widget_get_type (void);
 #define FLEGITA_OUTPUT_FILENAME_WIDGET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), FLEGITA_TYPE_OUTPUT_FILENAME_WIDGET, FlegitaOutputFilenameWidgetPrivate))
 enum  {
 	FLEGITA_OUTPUT_FILENAME_WIDGET_DUMMY_PROPERTY
 };
+GType flegita_option_output_filename_get_type (void);
+void flegita_option_output_filename_set_value (FlegitaOptionOutputFilename* self, const char* value);
 static void flegita_output_filename_widget_update_filename (FlegitaOutputFilenameWidget* self);
+const char* flegita_option_output_filename_get_value (FlegitaOptionOutputFilename* self);
 static void flegita_output_filename_widget_parse_filename (FlegitaOutputFilenameWidget* self);
+FlegitaOutputFilenameWidget* flegita_output_filename_widget_new (void);
+FlegitaOutputFilenameWidget* flegita_output_filename_widget_construct (GType object_type);
 static void _flegita_output_filename_widget_parse_filename_g_object_notify (GnomeScanOption* _sender, GParamSpec* pspec, gpointer self);
+GSList* flegita_option_output_filename_list_formats (FlegitaOptionOutputFilename* self);
 static void _flegita_output_filename_widget_update_filename_gtk_file_chooser_selection_changed (GtkFileChooser* _sender, gpointer self);
 static void _flegita_output_filename_widget_update_filename_gtk_editable_changed (GtkEntry* _sender, gpointer self);
 static void _flegita_output_filename_widget_update_filename_gtk_combo_box_changed (GtkComboBox* _sender, gpointer self);
 static GObject * flegita_output_filename_widget_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
-static gpointer flegita_output_filename_widget_parent_class = NULL;
 static void flegita_output_filename_widget_finalize (GObject* obj);
+static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
 static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
+static gint _vala_array_length (gpointer array);
 
 
 
@@ -77,111 +120,108 @@ GType flegita_format_columns_get_type (void) {
 }
 
 
+static gpointer _g_object_ref0 (gpointer self) {
+	return self ? g_object_ref (self) : NULL;
+}
+
+
 static void flegita_output_filename_widget_update_filename (FlegitaOutputFilenameWidget* self) {
-	FlegitaOptionOutputFilename* _tmp1;
-	GnomeScanOption* _tmp0;
+	GnomeScanOption* _tmp0_;
 	FlegitaOptionOutputFilename* option;
 	char* suffix;
 	GtkTreeIter iter = {0};
 	gboolean format_selected;
-	char* _tmp2;
+	char* _tmp2_;
+	char* _tmp1_;
 	g_return_if_fail (self != NULL);
 	if (self->priv->inhibit) {
 		return;
 	}
-	_tmp1 = NULL;
-	_tmp0 = NULL;
-	option = (_tmp1 = (_tmp0 = gnome_scan_option_widget_get_option ((GnomeScanOptionWidget*) self), FLEGITA_IS_OPTION_OUTPUT_FILENAME (_tmp0) ? ((FlegitaOptionOutputFilename*) _tmp0) : NULL), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
+	option = _g_object_ref0 ((_tmp0_ = gnome_scan_option_widget_get_option ((GnomeScanOptionWidget*) self), FLEGITA_IS_OPTION_OUTPUT_FILENAME (_tmp0_) ? ((FlegitaOptionOutputFilename*) _tmp0_) : NULL));
 	suffix = NULL;
 	format_selected = gtk_combo_box_get_active_iter (self->priv->combobox, &iter);
 	g_return_if_fail (format_selected);
 	gtk_tree_model_get ((GtkTreeModel*) self->priv->formats, &iter, FLEGITA_FORMAT_COLUMNS_SUFFIX, &suffix, -1);
 	self->priv->inhibit = TRUE;
-	_tmp2 = NULL;
-	flegita_option_output_filename_set_value (option, _tmp2 = g_strdup_printf ("%s%c%s.%s", gtk_file_chooser_get_filename (self->priv->filechooser), (gint) G_DIR_SEPARATOR, gtk_entry_get_text (self->priv->entry), suffix));
-	_tmp2 = (g_free (_tmp2), NULL);
+	flegita_option_output_filename_set_value (option, _tmp2_ = g_strdup_printf ("%s%c%s.%s", _tmp1_ = gtk_file_chooser_get_filename (self->priv->filechooser), (gint) G_DIR_SEPARATOR, gtk_entry_get_text (self->priv->entry), suffix));
+	_g_free0 (_tmp2_);
+	_g_free0 (_tmp1_);
 	self->priv->inhibit = FALSE;
-	(option == NULL) ? NULL : (option = (g_object_unref (option), NULL));
-	suffix = (g_free (suffix), NULL);
+	_g_object_unref0 (option);
+	_g_free0 (suffix);
 }
 
 
 static void flegita_output_filename_widget_parse_filename (FlegitaOutputFilenameWidget* self) {
 	GtkTreeIter iter = {0};
-	FlegitaOptionOutputFilename* _tmp1;
-	GnomeScanOption* _tmp0;
+	GnomeScanOption* _tmp0_;
 	FlegitaOptionOutputFilename* option;
-	const char* _tmp2;
 	char* filename;
-	gboolean _tmp3;
-	char* _tmp4;
+	gboolean _tmp1_;
+	char* _tmp2_;
 	char* basename;
-	const char* _tmp5;
 	char* suffix;
-	const char* _tmp6;
 	char* path_string;
-	char** _tmp7;
+	char** _tmp4_;
+	gint parts_size;
 	gint parts_length1;
+	char** _tmp3_;
 	char** parts;
 	g_return_if_fail (self != NULL);
 	if (self->priv->inhibit) {
 		return;
 	}
 	self->priv->inhibit = TRUE;
-	_tmp1 = NULL;
-	_tmp0 = NULL;
-	option = (_tmp1 = (_tmp0 = gnome_scan_option_widget_get_option ((GnomeScanOptionWidget*) self), FLEGITA_IS_OPTION_OUTPUT_FILENAME (_tmp0) ? ((FlegitaOptionOutputFilename*) _tmp0) : NULL), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
-	_tmp2 = NULL;
-	filename = (_tmp2 = flegita_option_output_filename_get_value (option), (_tmp2 == NULL) ? NULL : g_strdup (_tmp2));
-	_tmp3 = FALSE;
+	option = _g_object_ref0 ((_tmp0_ = gnome_scan_option_widget_get_option ((GnomeScanOptionWidget*) self), FLEGITA_IS_OPTION_OUTPUT_FILENAME (_tmp0_) ? ((FlegitaOptionOutputFilename*) _tmp0_) : NULL));
+	filename = g_strdup (flegita_option_output_filename_get_value (option));
+	_tmp1_ = FALSE;
 	if (filename == NULL) {
-		_tmp3 = TRUE;
+		_tmp1_ = TRUE;
 	} else {
-		_tmp3 = g_utf8_strlen (filename, -1) == 0;
+		_tmp1_ = g_utf8_strlen (filename, -1) == 0;
 	}
-	if (_tmp3) {
-		(option == NULL) ? NULL : (option = (g_object_unref (option), NULL));
-		filename = (g_free (filename), NULL);
+	if (_tmp1_) {
+		_g_object_unref0 (option);
+		_g_free0 (filename);
 		return;
 	}
-	_tmp4 = NULL;
-	gtk_file_chooser_set_current_folder (self->priv->filechooser, _tmp4 = g_path_get_dirname (filename));
-	_tmp4 = (g_free (_tmp4), NULL);
+	gtk_file_chooser_set_current_folder (self->priv->filechooser, _tmp2_ = g_path_get_dirname (filename));
+	_g_free0 (_tmp2_);
 	basename = g_path_get_basename (filename);
-	_tmp5 = NULL;
-	suffix = (_tmp5 = g_utf8_strrchr (basename, (glong) (-1), (gunichar) '.'), (_tmp5 == NULL) ? NULL : g_strdup (_tmp5));
+	suffix = g_strdup (g_utf8_strrchr (basename, (gssize) (-1), (gunichar) '.'));
 	if (suffix == NULL) {
 		g_warning ("flegita-option-widgets.vala:129: unable to parse suffix in %s", basename);
-		(option == NULL) ? NULL : (option = (g_object_unref (option), NULL));
-		filename = (g_free (filename), NULL);
-		basename = (g_free (basename), NULL);
-		suffix = (g_free (suffix), NULL);
+		_g_object_unref0 (option);
+		_g_free0 (filename);
+		_g_free0 (basename);
+		_g_free0 (suffix);
 		return;
 	}
-	_tmp6 = NULL;
-	path_string = (_tmp6 = (const char*) g_hash_table_lookup (self->priv->format_paths, suffix), (_tmp6 == NULL) ? NULL : g_strdup (_tmp6));
+	path_string = g_strdup ((const char*) g_hash_table_lookup (self->priv->format_paths, suffix));
 	if (!gtk_tree_model_get_iter_from_string ((GtkTreeModel*) self->priv->formats, &iter, path_string)) {
 		g_warning ("flegita-option-widgets.vala:135: Invalid %s suffix", suffix);
-		(option == NULL) ? NULL : (option = (g_object_unref (option), NULL));
-		filename = (g_free (filename), NULL);
-		basename = (g_free (basename), NULL);
-		suffix = (g_free (suffix), NULL);
-		path_string = (g_free (path_string), NULL);
+		_g_object_unref0 (option);
+		_g_free0 (filename);
+		_g_free0 (basename);
+		_g_free0 (suffix);
+		_g_free0 (path_string);
 		return;
 	}
 	gtk_combo_box_set_active_iter (self->priv->combobox, &iter);
-	_tmp7 = NULL;
-	parts = (_tmp7 = g_strsplit (basename, suffix, -1), parts_length1 = -1, _tmp7);
+	parts = (_tmp4_ = _tmp3_ = g_strsplit (basename, suffix, -1), parts_length1 = _vala_array_length (_tmp3_), parts_size = parts_length1, _tmp4_);
 	gtk_entry_set_text (self->priv->entry, parts[0]);
-	while (gdk_events_pending ()) {
+	while (TRUE) {
+		if (!gdk_events_pending ()) {
+			break;
+		}
 		gtk_main_iteration ();
 	}
 	self->priv->inhibit = FALSE;
-	(option == NULL) ? NULL : (option = (g_object_unref (option), NULL));
-	filename = (g_free (filename), NULL);
-	basename = (g_free (basename), NULL);
-	suffix = (g_free (suffix), NULL);
-	path_string = (g_free (path_string), NULL);
+	_g_object_unref0 (option);
+	_g_free0 (filename);
+	_g_free0 (basename);
+	_g_free0 (suffix);
+	_g_free0 (path_string);
 	parts = (_vala_array_free (parts, parts_length1, (GDestroyNotify) g_free), NULL);
 }
 
@@ -229,79 +269,62 @@ static GObject * flegita_output_filename_widget_constructor (GType type, guint n
 	self = FLEGITA_OUTPUT_FILENAME_WIDGET (obj);
 	{
 		GtkCellRenderer* renderer;
-		FlegitaOptionOutputFilename* _tmp0;
 		FlegitaOptionOutputFilename* option;
 		GtkTreeIter iter = {0};
 		GtkBox* box;
-		GtkFileChooser* _tmp1;
-		GtkEntry* _tmp2;
-		GtkListStore* _tmp3;
-		GHashTable* _tmp4;
-		GtkComboBox* _tmp7;
-		GtkCellRenderer* _tmp8;
-		GtkCellRenderer* _tmp9;
+		GtkFileChooser* _tmp0_;
+		GtkEntry* _tmp1_;
+		GtkListStore* _tmp2_;
+		GHashTable* _tmp3_;
+		GtkComboBox* _tmp4_;
+		GtkCellRenderer* _tmp5_;
+		GtkCellRenderer* _tmp6_;
 		renderer = NULL;
-		_tmp0 = NULL;
-		option = (_tmp0 = FLEGITA_OPTION_OUTPUT_FILENAME (gnome_scan_option_widget_get_option ((GnomeScanOptionWidget*) self)), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
+		option = _g_object_ref0 (FLEGITA_OPTION_OUTPUT_FILENAME (gnome_scan_option_widget_get_option ((GnomeScanOptionWidget*) self)));
 		gnome_scan_option_widget_set_no_label ((GnomeScanOptionWidget*) self, TRUE);
 		gnome_scan_option_widget_set_expand ((GnomeScanOptionWidget*) self, FALSE);
 		g_signal_connect_object ((GObject*) gnome_scan_option_widget_get_option ((GnomeScanOptionWidget*) self), "notify::value", (GCallback) _flegita_output_filename_widget_parse_filename_g_object_notify, self, 0);
 		box = (GtkBox*) g_object_ref_sink ((GtkHBox*) gtk_hbox_new (FALSE, 6));
 		gtk_box_pack_start ((GtkBox*) self, (GtkWidget*) box, TRUE, TRUE, (guint) 0);
-		/* directory selector*/
-		_tmp1 = NULL;
-		self->priv->filechooser = (_tmp1 = (GtkFileChooser*) g_object_ref_sink ((GtkFileChooserButton*) gtk_file_chooser_button_new (_ ("Select output directory"), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER)), (self->priv->filechooser == NULL) ? NULL : (self->priv->filechooser = (g_object_unref (self->priv->filechooser), NULL)), _tmp1);
+		self->priv->filechooser = (_tmp0_ = (GtkFileChooser*) g_object_ref_sink ((GtkFileChooserButton*) gtk_file_chooser_button_new (_ ("Select output directory"), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER)), _g_object_unref0 (self->priv->filechooser), _tmp0_);
 		gtk_box_pack_start (box, (GtkWidget*) self->priv->filechooser, TRUE, TRUE, (guint) 0);
-		/* filename entry*/
-		_tmp2 = NULL;
-		self->priv->entry = (_tmp2 = g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), (self->priv->entry == NULL) ? NULL : (self->priv->entry = (g_object_unref (self->priv->entry), NULL)), _tmp2);
+		self->priv->entry = (_tmp1_ = g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), _g_object_unref0 (self->priv->entry), _tmp1_);
 		gtk_box_pack_start (box, (GtkWidget*) self->priv->entry, TRUE, TRUE, (guint) 0);
-		/* format store*/
-		_tmp3 = NULL;
-		self->priv->formats = (_tmp3 = gtk_list_store_new ((gint) FLEGITA_FORMAT_COLUMNS_N, G_TYPE_STRING, G_TYPE_STRING, NULL), (self->priv->formats == NULL) ? NULL : (self->priv->formats = (g_object_unref (self->priv->formats), NULL)), _tmp3);
-		_tmp4 = NULL;
-		self->priv->format_paths = (_tmp4 = g_hash_table_new (g_str_hash, g_str_equal), (self->priv->format_paths == NULL) ? NULL : (self->priv->format_paths = (g_hash_table_unref (self->priv->format_paths), NULL)), _tmp4);
+		self->priv->formats = (_tmp2_ = gtk_list_store_new ((gint) FLEGITA_FORMAT_COLUMNS_N, G_TYPE_STRING, G_TYPE_STRING, NULL), _g_object_unref0 (self->priv->formats), _tmp2_);
+		self->priv->format_paths = (_tmp3_ = g_hash_table_new (g_str_hash, g_str_equal), _g_hash_table_unref0 (self->priv->format_paths), _tmp3_);
 		{
 			GSList* format_collection;
 			GSList* format_it;
 			format_collection = flegita_option_output_filename_list_formats (option);
 			for (format_it = format_collection; format_it != NULL; format_it = format_it->next) {
-				GnomeScanFormat* _tmp6;
 				GnomeScanFormat* format;
-				_tmp6 = NULL;
-				format = (_tmp6 = (GnomeScanFormat*) format_it->data, (_tmp6 == NULL) ? NULL : g_object_ref (_tmp6));
+				format = _g_object_ref0 ((GnomeScanFormat*) format_it->data);
 				{
 					GtkTreePath* path;
-					const char* _tmp5;
 					gtk_list_store_append (self->priv->formats, &iter);
 					gtk_list_store_set (self->priv->formats, &iter, FLEGITA_FORMAT_COLUMNS_ICON, format->icon_name, FLEGITA_FORMAT_COLUMNS_SUFFIX, format->suffixes[0], -1);
 					path = gtk_tree_model_get_path ((GtkTreeModel*) self->priv->formats, &iter);
-					_tmp5 = NULL;
-					g_hash_table_insert (self->priv->format_paths, g_strdup_printf (".%s", format->suffixes[0]), (_tmp5 = gtk_tree_path_to_string (path), (_tmp5 == NULL) ? NULL : g_strdup (_tmp5)));
-					(format == NULL) ? NULL : (format = (g_object_unref (format), NULL));
-					(path == NULL) ? NULL : (path = (gtk_tree_path_free (path), NULL));
+					g_hash_table_insert (self->priv->format_paths, g_strdup_printf (".%s", format->suffixes[0]), g_strdup (gtk_tree_path_to_string (path)));
+					_g_object_unref0 (format);
+					_gtk_tree_path_free0 (path);
 				}
 			}
 		}
-		/* format selector*/
-		_tmp7 = NULL;
-		self->priv->combobox = (_tmp7 = g_object_ref_sink ((GtkComboBox*) gtk_combo_box_new_with_model ((GtkTreeModel*) self->priv->formats)), (self->priv->combobox == NULL) ? NULL : (self->priv->combobox = (g_object_unref (self->priv->combobox), NULL)), _tmp7);
+		self->priv->combobox = (_tmp4_ = g_object_ref_sink ((GtkComboBox*) gtk_combo_box_new_with_model ((GtkTreeModel*) self->priv->formats)), _g_object_unref0 (self->priv->combobox), _tmp4_);
 		gtk_box_pack_start (box, (GtkWidget*) self->priv->combobox, FALSE, FALSE, (guint) 0);
-		_tmp8 = NULL;
-		renderer = (_tmp8 = (GtkCellRenderer*) g_object_ref_sink ((GtkCellRendererPixbuf*) gtk_cell_renderer_pixbuf_new ()), (renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL)), _tmp8);
+		renderer = (_tmp5_ = (GtkCellRenderer*) g_object_ref_sink ((GtkCellRendererPixbuf*) gtk_cell_renderer_pixbuf_new ()), _g_object_unref0 (renderer), _tmp5_);
 		gtk_cell_layout_pack_start ((GtkCellLayout*) self->priv->combobox, renderer, FALSE);
 		gtk_cell_layout_set_attributes ((GtkCellLayout*) self->priv->combobox, renderer, "icon-name", FLEGITA_FORMAT_COLUMNS_ICON, NULL);
-		_tmp9 = NULL;
-		renderer = (_tmp9 = (GtkCellRenderer*) g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), (renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL)), _tmp9);
+		renderer = (_tmp6_ = (GtkCellRenderer*) g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), _g_object_unref0 (renderer), _tmp6_);
 		gtk_cell_layout_pack_start ((GtkCellLayout*) self->priv->combobox, renderer, TRUE);
 		gtk_cell_layout_set_attributes ((GtkCellLayout*) self->priv->combobox, renderer, "text", FLEGITA_FORMAT_COLUMNS_SUFFIX, NULL);
 		g_signal_connect_object (self->priv->filechooser, "selection-changed", (GCallback) _flegita_output_filename_widget_update_filename_gtk_file_chooser_selection_changed, self, 0);
 		g_signal_connect_object ((GtkEditable*) self->priv->entry, "changed", (GCallback) _flegita_output_filename_widget_update_filename_gtk_editable_changed, self, 0);
 		g_signal_connect_object (self->priv->combobox, "changed", (GCallback) _flegita_output_filename_widget_update_filename_gtk_combo_box_changed, self, 0);
 		flegita_output_filename_widget_parse_filename (self);
-		(renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL));
-		(option == NULL) ? NULL : (option = (g_object_unref (option), NULL));
-		(box == NULL) ? NULL : (box = (g_object_unref (box), NULL));
+		_g_object_unref0 (renderer);
+		_g_object_unref0 (option);
+		_g_object_unref0 (box);
 	}
 	return obj;
 }
@@ -324,11 +347,11 @@ static void flegita_output_filename_widget_instance_init (FlegitaOutputFilenameW
 static void flegita_output_filename_widget_finalize (GObject* obj) {
 	FlegitaOutputFilenameWidget * self;
 	self = FLEGITA_OUTPUT_FILENAME_WIDGET (obj);
-	(self->priv->entry == NULL) ? NULL : (self->priv->entry = (g_object_unref (self->priv->entry), NULL));
-	(self->priv->filechooser == NULL) ? NULL : (self->priv->filechooser = (g_object_unref (self->priv->filechooser), NULL));
-	(self->priv->formats == NULL) ? NULL : (self->priv->formats = (g_object_unref (self->priv->formats), NULL));
-	(self->priv->combobox == NULL) ? NULL : (self->priv->combobox = (g_object_unref (self->priv->combobox), NULL));
-	(self->priv->format_paths == NULL) ? NULL : (self->priv->format_paths = (g_hash_table_unref (self->priv->format_paths), NULL));
+	_g_object_unref0 (self->priv->entry);
+	_g_object_unref0 (self->priv->filechooser);
+	_g_object_unref0 (self->priv->formats);
+	_g_object_unref0 (self->priv->combobox);
+	_g_hash_table_unref0 (self->priv->format_paths);
 	G_OBJECT_CLASS (flegita_output_filename_widget_parent_class)->finalize (obj);
 }
 
@@ -343,22 +366,35 @@ GType flegita_output_filename_widget_get_type (void) {
 }
 
 
-static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
+static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
 	if ((array != NULL) && (destroy_func != NULL)) {
 		int i;
-		if (array_length >= 0)
 		for (i = 0; i < array_length; i = i + 1) {
-			if (((gpointer*) array)[i] != NULL)
-			destroy_func (((gpointer*) array)[i]);
-		}
-		else
-		for (i = 0; ((gpointer*) array)[i] != NULL; i = i + 1) {
-			destroy_func (((gpointer*) array)[i]);
+			if (((gpointer*) array)[i] != NULL) {
+				destroy_func (((gpointer*) array)[i]);
+			}
 		}
 	}
+}
+
+
+static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
+	_vala_array_destroy (array, array_length, destroy_func);
 	g_free (array);
 }
 
 
+static gint _vala_array_length (gpointer array) {
+	int length;
+	length = 0;
+	if (array) {
+		while (((gpointer*) array)[length]) {
+			length++;
+		}
+	}
+	return length;
+}
+
+
 
 
